@@ -11,7 +11,7 @@ function ChooserWidget(id, opts) {
 
     this.id = id;
     this.chooserElement = $('#' + id + '-chooser');
-    this.titleElement = this.chooserElement.find('.title');
+    this.titleElement = this.chooserElement.find('[data-chooser-title]');
     this.inputElement = $('#' + id);
     this.editLinkElement = this.chooserElement.find('.edit-link');
     this.editLinkWrapper = this.chooserElement.find('.edit-link-wrapper');
@@ -20,15 +20,11 @@ function ChooserWidget(id, opts) {
     }
     this.chooseButton = $('.action-choose', this.chooserElement);
     this.idForLabel = null;
+    this.baseModalURL = opts.modalURL || this.chooserElement.data('choose-modal-url');
 
     this.modalResponses = {};
     this.modalResponses[opts.modalWorkflowResponseName || 'chosen'] = function(data) {
-        self.setState({
-            'value': data.id,
-            'title': data.string,
-            'edit_item_url': data.edit_link
-        });
-        self.inputElement.trigger('change');
+        self.setStateFromModalData(data);
     };
 
     this.chooseButton.on('click', function() {
@@ -38,10 +34,13 @@ function ChooserWidget(id, opts) {
     $('.action-clear', this.chooserElement).on('click', function() {
         self.setState(null);
     });
+
+    // attach a reference to this widget object onto the root element of the chooser
+    this.chooserElement.get(0).widget = this;
 }
 
 ChooserWidget.prototype.getModalURL = function() {
-    return this.chooserElement.data('choose-modal-url');
+    return this.baseModalURL;
 };
 
 ChooserWidget.prototype.openModal = function() {
@@ -52,6 +51,14 @@ ChooserWidget.prototype.openModal = function() {
     });
 };
 
+ChooserWidget.prototype.setStateFromModalData = function(data) {
+    this.setState({
+        'value': data.id,
+        'title': data.string,
+        'edit_item_url': data.edit_link
+    });
+}
+
 ChooserWidget.prototype.setState = function(newState) {
     if (newState && newState.value !== null && newState.value !== '') {
         this.inputElement.val(newState.value);
@@ -67,6 +74,7 @@ ChooserWidget.prototype.setState = function(newState) {
         this.inputElement.val('');
         this.chooserElement.addClass('blank');
     }
+    this.inputElement.trigger('change');
 };
 
 ChooserWidget.prototype.getState = function() {
@@ -84,3 +92,35 @@ ChooserWidget.prototype.getValue = function() {
 ChooserWidget.prototype.focus = function() {
     this.chooseButton.focus();
 }
+
+
+function ChooserWidgetFactory(html, opts) {
+    this.html = html;
+    this.opts = opts;
+    this.widgetClass = ChooserWidget;
+}
+ChooserWidgetFactory.prototype.render = function(placeholder, name, id, initialState) {
+    var html = this.html.replace(/__NAME__/g, name).replace(/__ID__/g, id);
+    placeholder.outerHTML = html;
+
+    var chooser = new this.widgetClass(id, this.opts);
+    chooser.setState(initialState);
+    return chooser;
+};
+ChooserWidgetFactory.prototype.openModal = function(callback, urlParams) {
+    var responses = [];
+    responses[this.opts.modalWorkflowResponseName || 'chosen'] = callback;
+
+    ModalWorkflow({
+        url: this.opts.modalURL,
+        urlParams: urlParams,
+        onload: GENERIC_CHOOSER_MODAL_ONLOAD_HANDLERS,
+        responses: responses
+    });
+};
+ChooserWidgetFactory.prototype.getById = function(id) {
+    /* retrieve the widget object corresponding to the given HTML ID */
+    return document.getElementById(id + '-chooser').widget;
+}
+
+window.WagtailGenericChooserWidgetFactory = ChooserWidgetFactory;
